/*
 * Copyright (C) 1994-2020 Altair Engineering, Inc.
 * For more information, contact Altair at www.altair.com.
 *
 * This file is part of both the OpenPBS software ("OpenPBS")
 * and the PBS Professional ("PBS Pro") software.
 *
 * Open Source License Information:
 *
 * OpenPBS is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * OpenPBS is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Commercial License Information:
 *
 * PBS Pro is commercially licensed software that shares a common core with
 * the OpenPBS software.  For a copy of the commercial license terms and
 * conditions, go to: (http://www.pbspro.com/agreement.html) or contact the
 * Altair Legal Department.
 *
 * Altair's dual-license business model allows companies, individuals, and
 * organizations to create proprietary derivative works of OpenPBS and
 * distribute them - whether embedded or bundled with other software -
 * under a commercial license agreement.
 *
 * Use of Altair's trademarks, including but not limited to "PBS™",
 * "OpenPBS®", "PBS Professional®", and "PBS Pro™" and Altair's logos is
 * subject to Altair's trademark licensing policies.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <float.h>
#include <stddef.h>
#include <stdlib.h>

#include <dis.h>
/**
 * @file	disi10l_.c
 */
unsigned dis_lmx10 = 0;
dis_long_double_t *dis_lp10 = NULL;
dis_long_double_t *dis_ln10 = NULL;

/**
 * @brief
 *	-Allocate and fill tables with all powers of 10 that fit the forms:
 *
 *				  n
 *				 2
 *		*dis_lp10[n] = 10
 *
 *				  ( n)
 *				 -(2 )
 *		*dis_ln10[n] = 10
 *
 *	For all values of n supported by the floating point format.  Set
 *	dis_lmx10 equal to the largest value of n that fits the format.
 */

void
disi10l_()
{
	int		i;
	unsigned long	ul;
	dis_long_double_t	accum;
	size_t		tabsize;

	assert(dis_lp10 == NULL);
	assert(dis_ln10 == NULL);
	assert(dis_lmx10 == 0);

#if LDBL_MAX_10_EXP + LDBL_MIN_10_EXP > 0
	ul = LDBL_MAX_10_EXP;
#else
	ul = -LDBL_MIN_10_EXP;
#endif
	while (ul >>= 1)
		dis_lmx10++;
	tabsize = (dis_lmx10 + 1) * sizeof(dis_long_double_t);
	dis_lp10 = (dis_long_double_t *)malloc(tabsize);
	assert(dis_lp10 != NULL);
	dis_ln10 = (dis_long_double_t *)malloc(tabsize);
	assert(dis_ln10 != NULL);
	assert(dis_lmx10 > 0);
	dis_lp10[0] = accum = 10.0L;
	dis_ln10[0] = 1.0L / accum;
	for (i = 1; i <= dis_lmx10; i++) {
		accum *= accum;
		dis_lp10[i] = accum;
		dis_ln10[i] = 1.0L / accum;
	}
}
