/*
 * Copyright (C) 1994-2020 Altair Engineering, Inc.
 * For more information, contact Altair at www.altair.com.
 *
 * This file is part of both the OpenPBS software ("OpenPBS")
 * and the PBS Professional ("PBS Pro") software.
 *
 * Open Source License Information:
 *
 * OpenPBS is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * OpenPBS is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Commercial License Information:
 *
 * PBS Pro is commercially licensed software that shares a common core with
 * the OpenPBS software.  For a copy of the commercial license terms and
 * conditions, go to: (http://www.pbspro.com/agreement.html) or contact the
 * Altair Legal Department.
 *
 * Altair's dual-license business model allows companies, individuals, and
 * organizations to create proprietary derivative works of OpenPBS and
 * distribute them - whether embedded or bundled with other software -
 * under a commercial license agreement.
 *
 * Use of Altair's trademarks, including but not limited to "PBS™",
 * "OpenPBS®", "PBS Professional®", and "PBS Pro™" and Altair's logos is
 * subject to Altair's trademark licensing policies.
 */

/**
 * @file	PBS_attr.c
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <stdio.h>
#include <stdlib.h>
#include "libpbs.h"


/**
 * @brief
 *	-parses attr list and checks if name and value are null.
 *
 * @param[in] alp - pointer to attr list
 *
 * @return	int
 * @retval	0	Success
 * @retval	-1	if val or name name
 *
 */

int
PBS_val_al(struct attrl *alp)
{
	while (alp) {
		if ((alp->name == 0) || (alp->value == 0))
			return -1;
		alp = alp->next;
	}
	return 0;
}

/**
 * @brief
 *	-frees the attr list
 *
 * @param[in] alp - pointer to attr list
 *
 * @return	Void
 *
 */
void
PBS_free_al(struct attrl *alp)
{
	struct attrl *talp;
	while (alp != NULL) {
		free(alp->name);
		free(alp->resource);
		free(alp->value);
		talp = alp;
		alp = alp->next;
		free(talp);
	}
}

/**
 * @brief
 *      -parses attr list with option and checks if name and value are null.
 *
 * @param[in] alp - pointer to attr list
 *
 * @return      int
 * @retval      0       Success
 * @retval      -1      if val or name name
 *
 */

int
PBS_val_aopl(struct attropl *aoplp)
{
	while (aoplp != NULL) {
		if ((aoplp->name == 0) || (aoplp == 0))
			return -1;
		aoplp = aoplp->next;
	}
	return 0;
}

/**
 * @brief
 *      -frees the attr list with option
 *
 * @param[in] alp - pointer to attr list
 *
 * @return      Void
 *
 */

void
PBS_free_aopl(struct attropl *aoplp)
{
	struct attropl *taoplp;
	while (aoplp != NULL) {
		if (aoplp->name) {
			free(aoplp->name);
			aoplp->name = NULL;
		}
		if (aoplp->resource) {
			free(aoplp->resource);
			aoplp->resource = NULL;
		}
		if (aoplp->value) {
			free(aoplp->value);
			aoplp->value = NULL;
		}
		taoplp = aoplp;
		aoplp = aoplp->next;
		if (taoplp) {
			free(taoplp);
			taoplp = NULL;
		}
	}
}
