/*
 * Copyright (C) 1994-2020 Altair Engineering, Inc.
 * For more information, contact Altair at www.altair.com.
 *
 * This file is part of both the OpenPBS software ("OpenPBS")
 * and the PBS Professional ("PBS Pro") software.
 *
 * Open Source License Information:
 *
 * OpenPBS is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * OpenPBS is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Commercial License Information:
 *
 * PBS Pro is commercially licensed software that shares a common core with
 * the OpenPBS software.  For a copy of the commercial license terms and
 * conditions, go to: (http://www.pbspro.com/agreement.html) or contact the
 * Altair Legal Department.
 *
 * Altair's dual-license business model allows companies, individuals, and
 * organizations to create proprietary derivative works of OpenPBS and
 * distribute them - whether embedded or bundled with other software -
 * under a commercial license agreement.
 *
 * Use of Altair's trademarks, including but not limited to "PBS™",
 * "OpenPBS®", "PBS Professional®", and "PBS Pro™" and Altair's logos is
 * subject to Altair's trademark licensing policies.
 */

/**
 * @file	get_svrport.c
 */
#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include <netdb.h>
#include <netinet/in.h>
#include "portability.h"

/*
 * get_svrport - get the port number for a given service
 *
 *	Return the port number in an unsigned interger in host byte order.
 *	Assumes the protocol type is tcp.
 *	Returns 0 on an error.
 */


/**
 * @brief
 *	-get the port number for a given service
 *
 * @param[in] service_name - service for which port reqd
 * @param[in] ptype - port type
 * @param[in] pdefault - in host byte order
 *
 * @return	unsigned int
 * @retval	port num	success
 * @retval	0		error
 *
 */
unsigned int
get_svrport(char *service_name, char *ptype, unsigned int pdefault)
{
	struct servent *psvr;

	psvr = getservbyname(service_name, ptype);
	if (psvr)
		return ((unsigned int)ntohs(psvr->s_port));
	else
		return (pdefault);
}
