/*
 * Copyright (C) 1994-2020 Altair Engineering, Inc.
 * For more information, contact Altair at www.altair.com.
 *
 * This file is part of both the OpenPBS software ("OpenPBS")
 * and the PBS Professional ("PBS Pro") software.
 *
 * Open Source License Information:
 *
 * OpenPBS is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * OpenPBS is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Commercial License Information:
 *
 * PBS Pro is commercially licensed software that shares a common core with
 * the OpenPBS software.  For a copy of the commercial license terms and
 * conditions, go to: (http://www.pbspro.com/agreement.html) or contact the
 * Altair Legal Department.
 *
 * Altair's dual-license business model allows companies, individuals, and
 * organizations to create proprietary derivative works of OpenPBS and
 * distribute them - whether embedded or bundled with other software -
 * under a commercial license agreement.
 *
 * Use of Altair's trademarks, including but not limited to "PBS™",
 * "OpenPBS®", "PBS Professional®", and "PBS Pro™" and Altair's logos is
 * subject to Altair's trademark licensing policies.
 */

/**
 * @file	pbs_geterrmsg.c
 * @brief
 * Return the last error message the server returned on
 * this connection.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include "libpbs.h"


/**
 * @brief
 *	-Return the last error message the server returned on
 *	this connection.
 *
 * @param[in] connect - soket descriptor
 *
 * @return	string
 * @retval	connection contexts
 *		TLS			multithread
 *		STRUCTURE		single thread
 * @retval	errmsg			error
 *
 */
char *
__pbs_geterrmsg(int connect)
{
	struct pbs_client_thread_connect_context * con =
		pbs_client_thread_find_connect_context(connect);
	struct pbs_client_thread_context *thrd_ctxt = pbs_client_thread_get_context_data();

	/*
	 * multithreaded callers will have the connection contexts stored
	 * in the TLS, whereas single threaded clients dont use the TLS
	 * So, return connection structure values after checking
	 */
	if (con && thrd_ctxt && (thrd_ctxt->th_pbs_mode==0))
		return (con->th_ch_errtxt);
	else
		return get_conn_errtxt(connect);
}
